@@ -8,6 +8,26 @@
 
 #include <stdlib.h>
 #include <stdio.h>
+#include <assert.h>
+
+#if defined (HAVE_EXECINFO_H)
+#include <execinfo.h>
+#endif
+
+static
+void show_stackframe()
+{
+#if defined (HAVE_EXECINFO_H)
+  void *trace[16];
+  size_t i;
+  size_t trace_size = backtrace(trace, 16);
+  char **messages = backtrace_symbols(trace, trace_size);
+
+  fprintf(stderr, "[bt] Execution path:\n");
+  for ( i = 0; i < trace_size; ++i ) fprintf(stderr, "[bt] %s\n", messages[i]);
+  if ( messages ) free(messages);
+#endif
+}
 
 #include "dmemory.h"
 #include "resource_handle.h"
@@ -17,12 +37,40 @@
 #include "error.h"
 #include "file.h"
 #include "resource_unpack.h"
+#include "institution.h"
+#include "model.h"
 
 enum { MIN_LIST_SIZE = 128 };
 
 static void listInitialize(void);
 
-// ATTENTION: not thread safe yet, namespaces are set in model!
+typedef struct listElem {
+  union
+  {
+    /* free-list management data */
+    struct
+    {
+      int next, prev;
+    } free;
+    /* holding an actual value */
+    struct
+    {
+      const resOps *ops;
+      void         *val;//ptr
+    } v;
+  } res;
+  int           status;
+} listElem_t;
+
+struct resHList_t
+{
+  int size, freeHead, hasDefaultRes;
+  listElem_t *resources;
+};
+
+static struct resHList_t *resHList;
+
+static int resHListSize = 0;
 
 #if  defined  (HAVE_LIBPTHREAD)
 #  include <pthread.h>
@@ -32,7 +80,15 @@ static pthread_mutex_t listMutex;
 
 #  define LIST_LOCK()         pthread_mutex_lock(&listMutex)
 #  define LIST_UNLOCK()       pthread_mutex_unlock(&listMutex)
-#  define LIST_INIT()         pthread_once(&listInitThread, listInitialize)
+#  define LIST_INIT(init0)         do {                         \
+    pthread_once(&listInitThread, listInitialize);              \
+    pthread_mutex_lock(&listMutex);                             \
+    if ((init0) && (!resHList || !resHList[0].resources))       \
+      reshListCreate(0);                                        \
+    pthread_mutex_unlock(&listMutex);                           \
+  } while (0)
+
+
 
 #else
 
@@ -40,33 +96,18 @@ static int listInit = 0;
 
 #  define LIST_LOCK()
 #  define LIST_UNLOCK()
-#  define LIST_INIT()        do {                               \
+#  define LIST_INIT(init0)        do {                          \
   if ( !listInit )                                              \
     {                                                           \
       listInitialize();                                         \
+      if ((init0) && (!resHList || !resHList[0].resources))     \
+        reshListCreate(0);                                      \
       listInit = 1;                                             \
     }                                                           \
   } while(0)
 
 #endif
 
-
-typedef struct listElem {
-  cdiResH       resH;//idx
-  int next;
-  resOps      * ops;
-  void        * val;//ptr
-  int           status;
-} listElem_t;
-
-static struct
-{
-  int size, freeHead;
-  listElem_t *resources;
-} *resHList;
-
-static int resHListSize = 0;
-
 /**************************************************************/
 
 static void
@@ -75,20 +116,23 @@ listInitResources(int nsp)
   xassert(nsp < resHListSize && nsp >= 0);
   int size = resHList[nsp].size = MIN_LIST_SIZE;
   xassert(resHList[nsp].resources == NULL);
-  resHList[nsp].resources = xcalloc(MIN_LIST_SIZE, sizeof (listElem_t));
+  resHList[nsp].resources = (listElem_t*) xcalloc(MIN_LIST_SIZE, sizeof(listElem_t));
   listElem_t *p = resHList[nsp].resources;
 
   for (int i = 0; i < size; i++ )
     {
-      p[i].resH   = namespaceIdxEncode2(nsp, i);
-      p[i].next   = i + 1;
-      p[i].ops    = NULL;
-      p[i].val    = NULL;
-      p[i].status = RESH_UNDEFID;
+      p[i].res.free.next = i + 1;
+      p[i].res.free.prev = i - 1;
+      p[i].status = RESH_UNUSED;
     }
 
-  p[resHList[nsp].size-1].next = -1;
+  p[size-1].res.free.next = -1;
   resHList[nsp].freeHead = 0;
+  int oldNsp = namespaceGetActive();
+  namespaceSetActive(nsp);
+  instituteDefaultEntries();
+  modelDefaultEntries();
+  namespaceSetActive(oldNsp);
 }
 
 static inline void
@@ -102,10 +146,11 @@ reshListClearEntry(int i)
 void
 reshListCreate(int namespaceID)
 {
+  LIST_INIT(namespaceID != 0);
   LIST_LOCK();
   if (resHListSize <= namespaceID)
     {
-      resHList = xrealloc(resHList, (namespaceID + 1) * sizeof (resHList[0]));
+      resHList = (struct resHList_t *)xrealloc(resHList, (namespaceID + 1) * sizeof (resHList[0]));
       for (int i = resHListSize; i <= namespaceID; ++i)
         reshListClearEntry(i);
       resHListSize = namespaceID + 1;
@@ -120,7 +165,6 @@ reshListCreate(int namespaceID)
 void
 reshListDestruct(int namespaceID)
 {
-  LIST_INIT();
   LIST_LOCK();
   xassert(resHList && namespaceID >= 0 && namespaceID < resHListSize);
   int callerNamespaceID = namespaceGetActive();
@@ -130,10 +174,11 @@ reshListDestruct(int namespaceID)
       for ( int j = 0; j < resHList[namespaceID].size; j++ )
         {
           listElem_t *listElem = resHList[namespaceID].resources + j;
-          if (listElem->val)
-            listElem->ops->valDestroy(listElem->val);
+          if (listElem->status & RESH_IN_USE_BIT)
+            listElem->res.v.ops->valDestroy(listElem->res.v.val);
         }
       free(resHList[namespaceID].resources);
+      resHList[namespaceID].resources = NULL;
       reshListClearEntry(namespaceID);
     }
   if (resHList[callerNamespaceID].resources)
@@ -145,9 +190,10 @@ reshListDestruct(int namespaceID)
 static void listDestroy ( void )
 {
   LIST_LOCK();
-  for (int i = 0; i < resHListSize; ++i)
-    if (resHList[i].resources)
-      namespaceDelete(i);
+  for (int i = resHListSize; i > 0; --i)
+    if (resHList[i-1].resources)
+      namespaceDelete(i-1);
+  resHListSize = 0;
   free(resHList);
   resHList = NULL;
   cdiReset();
@@ -167,8 +213,6 @@ void listInitialize ( void )
   pthread_mutex_init ( &listMutex, &ma);
   pthread_mutexattr_destroy(&ma);
 #endif
-  // create default namespace
-  reshListCreate(0);
   /* file is special and has its own table, which needs to be
    * created, before we register the listDestroy exit handler */
   {
@@ -178,7 +222,6 @@ void listInitialize ( void )
       fileClose_serial(null_id);
   }
   atexit ( listDestroy );
-
 }
 
 /**************************************************************/
@@ -188,49 +231,60 @@ void listSizeExtend()
 {
   int nsp = namespaceGetActive ();
   int oldSize = resHList[nsp].size;
-  int newListSize = oldSize + MIN_LIST_SIZE;
+  size_t newListSize = (size_t)oldSize + MIN_LIST_SIZE;
 
-  resHList[nsp].resources =
-    xrealloc(resHList[nsp].resources,
-             newListSize * sizeof (resHList[0].resources[0]));
+  resHList[nsp].resources = (listElem_t*) xrealloc(resHList[nsp].resources,
+                                                   newListSize * sizeof(listElem_t));
 
-  for (int i = oldSize; i < newListSize; ++i)
+  listElem_t *r = resHList[nsp].resources;
+  for (size_t i = (size_t)oldSize; i < newListSize; ++i)
     {
-      resHList[nsp].resources[i].resH   = namespaceIdxEncode2 ( nsp, i );
-      resHList[nsp].resources[i].next   = i + 1;
-      resHList[nsp].resources[i].ops    = NULL;
-      resHList[nsp].resources[i].val    = NULL;
-      resHList[nsp].resources[i].status = RESH_UNDEFID;
+      r[i].res.free.next = (int)i + 1;
+      r[i].res.free.prev = (int)i - 1;
+      r[i].status = RESH_UNUSED;
     }
 
-  resHList[nsp].resources[newListSize-1].next = resHList[nsp].freeHead;
+  if (resHList[nsp].freeHead != -1)
+    r[resHList[nsp].freeHead].res.free.prev = (int)newListSize - 1;
+  r[newListSize-1].res.free.next = resHList[nsp].freeHead;
+  r[oldSize].res.free.prev = -1;
   resHList[nsp].freeHead = oldSize;
-  resHList[nsp].size = newListSize;
+  resHList[nsp].size = (int)newListSize;
 }
 
 /**************************************************************/
 
-int reshPut ( void *p, resOps *ops )
+static void
+reshPut_(int nsp, int entry, void *p, const resOps *ops)
 {
-  cdiResH resH = -1, nsp;
-  listElem_t * newListElem;
+  listElem_t *newListElem = resHList[nsp].resources + entry;
+  int next = newListElem->res.free.next,
+    prev = newListElem->res.free.prev;
+  if (next != -1)
+    resHList[nsp].resources[next].res.free.prev = prev;
+  if (prev != -1)
+    resHList[nsp].resources[prev].res.free.next = next;
+  else
+    resHList[nsp].freeHead = next;
+  newListElem->res.v.val = p;
+  newListElem->res.v.ops = ops;
+  newListElem->status = RESH_DESYNC_IN_USE;
+}
 
+int reshPut ( void *p, const resOps *ops )
+{
   xassert ( p && ops );
 
-  LIST_INIT();
+  LIST_INIT(1);
 
   LIST_LOCK();
 
-  nsp = namespaceGetActive ();
+  int nsp = namespaceGetActive ();
 
   if ( resHList[nsp].freeHead == -1) listSizeExtend();
-  newListElem               = resHList[nsp].resources + resHList[nsp].freeHead;
-  resHList[nsp].freeHead    = newListElem->next;
-  newListElem->next         = -1;
-  resH                      = newListElem->resH;
-  newListElem->val          = p;
-  newListElem->ops          = ops;
-  newListElem->status       = ASSIGNED;
+  int entry = resHList[nsp].freeHead;
+  cdiResH resH = namespaceIdxEncode2(nsp, entry);
+  reshPut_(nsp, entry, p, ops);
 
   LIST_UNLOCK();
 
@@ -239,54 +293,104 @@ int reshPut ( void *p, resOps *ops )
 
 /**************************************************************/
 
-void reshRemove ( cdiResH resH, resOps * ops )
+static void
+reshRemove_(int nsp, int idx)
+{
+  int curFree = resHList[nsp].freeHead;
+  listElem_t *r = resHList[nsp].resources;
+  r[idx].res.free.next = curFree;
+  r[idx].res.free.prev = -1;
+  if (curFree != -1)
+    r[curFree].res.free.prev = idx;
+  r[idx].status = RESH_DESYNC_DELETED;
+  resHList[nsp].freeHead = idx;
+}
+
+void reshDestroy(cdiResH resH)
 {
   int nsp;
   namespaceTuple_t nspT;
 
-  LIST_INIT();
-
   LIST_LOCK();
 
   nsp = namespaceGetActive ();
 
   nspT = namespaceResHDecode ( resH );
 
-  xassert ( nspT.nsp == nsp &&
-            nspT.idx >= 0 &&
-            nspT.idx < resHList[nsp].size &&
-            resHList[nsp].resources[nspT.idx].ops &&
-            resHList[nsp].resources[nspT.idx].ops == ops );
+  xassert ( nspT.nsp == nsp
+            && nspT.idx >= 0
+            && nspT.idx < resHList[nsp].size
+            && resHList[nsp].resources[nspT.idx].res.v.ops);
 
-  resHList[nsp].resources[nspT.idx].next   = resHList[nsp].freeHead;
-  resHList[nsp].resources[nspT.idx].ops    = NULL;
-  resHList[nsp].resources[nspT.idx].val    = NULL;
-  resHList[nsp].resources[nspT.idx].status = RESH_UNDEFID;
-  resHList[nsp].freeHead                   = nspT.idx;
+  if (resHList[nsp].resources[nspT.idx].status & RESH_IN_USE_BIT)
+    reshRemove_(nsp, nspT.idx);
+
+  LIST_UNLOCK();
+}
+
+void reshRemove ( cdiResH resH, const resOps * ops )
+{
+  int nsp;
+  namespaceTuple_t nspT;
+
+  LIST_LOCK();
+
+  nsp = namespaceGetActive ();
+
+  nspT = namespaceResHDecode ( resH );
+
+  xassert ( nspT.nsp == nsp
+            && nspT.idx >= 0
+            && nspT.idx < resHList[nsp].size
+            && (resHList[nsp].resources[nspT.idx].status & RESH_IN_USE_BIT)
+            && resHList[nsp].resources[nspT.idx].res.v.ops
+            && resHList[nsp].resources[nspT.idx].res.v.ops == ops );
+
+  reshRemove_(nsp, nspT.idx);
 
   LIST_UNLOCK();
 }
 
 /**************************************************************/
 
+void reshReplace(cdiResH resH, void *p, const resOps *ops)
+{
+  xassert(p && ops);
+  LIST_INIT(1);
+  LIST_LOCK();
+  int nsp = namespaceGetActive();
+  namespaceTuple_t nspT = namespaceResHDecode(resH);
+  while (resHList[nsp].size <= nspT.idx)
+    listSizeExtend();
+  listElem_t *q = resHList[nsp].resources + nspT.idx;
+  if (q->status & RESH_IN_USE_BIT)
+    {
+      q->res.v.ops->valDestroy(q->res.v.val);
+      reshRemove_(nsp, nspT.idx);
+    }
+  reshPut_(nsp, nspT.idx, p, ops);
+  LIST_UNLOCK();
+}
+
+
 static listElem_t *
-reshGetElem(const char *caller, cdiResH resH, resOps *ops)
+reshGetElem(const char *caller, const char* expressionString, cdiResH resH, const resOps *ops)
 {
   listElem_t *listElem;
   int nsp;
   namespaceTuple_t nspT;
   xassert ( ops );
 
-  LIST_INIT();
+  LIST_INIT(1);
 
   LIST_LOCK();
 
   nsp = namespaceGetActive ();
 
   nspT = namespaceResHDecode ( resH );
+  assert(nspT.idx >= 0);
 
   if (nspT.nsp == nsp &&
-      nspT.idx >= 0 &&
       nspT.idx < resHList[nsp].size)
     {
       listElem = resHList[nsp].resources + nspT.idx;
@@ -295,43 +399,52 @@ reshGetElem(const char *caller, cdiResH resH, resOps *ops)
   else
     {
       LIST_UNLOCK();
-      xabortC(caller, "Invalid namespace %d or index %d for resource handle %d when using namespace %d of size %d!",
-              nspT.nsp, nspT.idx, (int)resH, nsp, resHList[nsp].size);
+      show_stackframe();
+
+      if ( resH == CDI_UNDEFID )
+        {
+          xabortC(caller, "Error while trying to resolve the ID \"%s\" in `%s()`: the value is CDI_UNDEFID (= %d).\n\tThis is most likely the result of a failed earlier call. Please check the IDs returned by CDI.", expressionString, caller, resH);
+        }
+      else
+        {
+          xabortC(caller, "Error while trying to resolve the ID \"%s\" in `%s()`: the value is garbage (= %d, which resolves to namespace = %d, index = %d).\n\tThis is either the result of using an uninitialized variable,\n\tof using a value as an ID that is not an ID,\n\tor of using an ID after it has been invalidated.", expressionString, caller, resH, nspT.nsp, nspT.idx);
+        }
+    }
+
+  if ( !(listElem && listElem->res.v.ops == ops) )
+    {
+      show_stackframe();
+
+      xabortC(caller, "Error while trying to resolve the ID \"%s\" in `%s()`: list element not found. The failed ID is %d", expressionString, caller, (int)resH);
     }
 
-  if ( !(listElem && listElem->ops == ops) )
-    xabortC(caller, "Invalid resource handle %d, list element not found!",
-            (int)resH);
   return listElem;
 }
 
 
-void *reshGetValue(const char * caller, cdiResH resH, resOps * ops)
+void *reshGetValue(const char * caller, const char* expressionString, cdiResH resH, const resOps * ops)
 {
-  return reshGetElem(caller, resH, ops)->val;
+  return reshGetElem(caller, expressionString, resH, ops)->res.v.val;
 }
 
 /**************************************************************/
 
-void reshGetResHListOfType ( int c, int * resHs, resOps * ops )
+void reshGetResHListOfType ( int c, int * resHs, const resOps * ops )
 {
   int i, j = 0, nsp;
 
   xassert ( resHs && ops );
 
-  LIST_INIT();
+  LIST_INIT(1);
 
   LIST_LOCK();
 
   nsp = namespaceGetActive ();
 
-  for ( i = 0; i < resHList[nsp].size; i++ )
-    if ( resHList[nsp].resources[i].val && resHList[nsp].resources[i].ops )
-      if ( resHList[nsp].resources[i].ops == ops )
-        {
-          resHs[j++] = namespaceIdxEncode2 ( nsp, i );
-          if ( j == c ) break;
-        }
+  for ( i = 0; i < resHList[nsp].size && j < c; i++ )
+    if ((resHList[nsp].resources[i].status & RESH_IN_USE_BIT)
+        && resHList[nsp].resources[i].res.v.ops == ops)
+      resHs[j++] = namespaceIdxEncode2(nsp, i);
 
   LIST_UNLOCK();
 }
@@ -342,16 +455,17 @@ cdiResHApply(enum cdiApplyRet (*func)(int id, void *res, const resOps *p,
 {
   xassert(func);
 
-  LIST_INIT();
+  LIST_INIT(1);
 
   LIST_LOCK();
 
   int nsp = namespaceGetActive ();
   enum cdiApplyRet ret = CDI_APPLY_GO_ON;
   for (int i = 0; i < resHList[nsp].size && ret > 0; ++i)
-    if (resHList[nsp].resources[i].val && resHList[nsp].resources[i].ops)
-      ret = func(namespaceIdxEncode2(nsp, i), resHList[nsp].resources[i].val,
-                 resHList[nsp].resources[i].ops, data);
+    if (resHList[nsp].resources[i].status & RESH_IN_USE_BIT)
+      ret = func(namespaceIdxEncode2(nsp, i),
+                 resHList[nsp].resources[i].res.v.val,
+                 resHList[nsp].resources[i].res.v.ops, data);
   LIST_UNLOCK();
   return ret;
 }
@@ -364,16 +478,16 @@ cdiResHFilterApply(const resOps *p,
 {
   xassert(p && func);
 
-  LIST_INIT();
+  LIST_INIT(1);
 
   LIST_LOCK();
 
   int nsp = namespaceGetActive ();
   enum cdiApplyRet ret = CDI_APPLY_GO_ON;
+  listElem_t *r = resHList[nsp].resources;
   for (int i = 0; i < resHList[nsp].size && ret > 0; ++i)
-    if (resHList[nsp].resources[i].val && resHList[nsp].resources[i].ops
-        && resHList[nsp].resources[i].ops == p)
-      ret = func(namespaceIdxEncode2(nsp, i), resHList[nsp].resources[i].val,
+    if ((r[i].status & RESH_IN_USE_BIT) && r[i].res.v.ops == p)
+      ret = func(namespaceIdxEncode2(nsp, i), r[i].res.v.val,
                  data);
   LIST_UNLOCK();
   return ret;
@@ -384,22 +498,21 @@ cdiResHFilterApply(const resOps *p,
 
 /**************************************************************/
 
-int reshCountType ( resOps * ops )
+int reshCountType ( const resOps * ops )
 {
   int i, nsp, countType = 0;
 
   xassert ( ops );
 
-  LIST_INIT();
+  LIST_INIT(1);
 
   LIST_LOCK();
 
   nsp = namespaceGetActive ();
 
+  listElem_t *r = resHList[nsp].resources;
   for ( i = 0; i < resHList[nsp].size; i++ )
-    if ( resHList[nsp].resources[i].val )
-      if ( resHList[nsp].resources[i].ops == ops )
-        countType++;
+    countType += ((r[i].status & RESH_IN_USE_BIT) && r[i].res.v.ops == ops);
 
   LIST_UNLOCK();
 
@@ -409,45 +522,48 @@ int reshCountType ( resOps * ops )
 /**************************************************************/
 
 int
-reshResourceGetPackSize(int resH, resOps *ops, void *context)
+reshResourceGetPackSize_intern(int resH, const resOps *ops, void *context, const char* caller, const char* expressionString)
 {
-  listElem_t *curr = reshGetElem(__func__, resH, ops);
-  return curr->ops->valGetPackSize(curr->val, context);
+  listElem_t *curr = reshGetElem(caller, expressionString, resH, ops);
+  return curr->res.v.ops->valGetPackSize(curr->res.v.val, context);
 }
 
 void
-reshPackResource(int resH, resOps *ops,
-                 void *buf, int buf_size, int *position, void *context)
+reshPackResource_intern(int resH, const resOps *ops, void *buf, int buf_size, int *position, void *context,
+                        const char* caller, const char* expressionString)
 {
-  listElem_t *curr = reshGetElem(__func__, resH, ops);
-  curr->ops->valPack(curr->val, buf, buf_size, position, context);
+  listElem_t *curr = reshGetElem(caller, expressionString, resH, ops);
+  curr->res.v.ops->valPack(curr->res.v.val, buf, buf_size, position, context);
 }
 
 
 static int getPackBufferSize(void *context)
 {
-  int nsp, i;
   int intpacksize, packBufferSize = 0;
-  listElem_t * curr;
 
-  nsp = namespaceGetActive ();
+  int nsp = namespaceGetActive ();
 
   /* pack start marker, namespace and sererator marker */
-  packBufferSize += (intpacksize = serializeGetSize(3, DATATYPE_INT, context));
+  packBufferSize += 3 * (intpacksize = serializeGetSize(1, DATATYPE_INT, context));
 
   /* pack resources, type marker and seperator marker */
-  for ( i = 0; i < resHList[nsp].size; i++ )
-    if ( resHList[nsp].resources[i].val )
-      if ( resHList[nsp].resources[i].status == ASSIGNED )
-        {
-          curr = resHList[nsp].resources + i;
-          xassert ( curr->ops );
-
-          /* message plus frame of 2 ints */
-          packBufferSize += curr->ops->valGetPackSize(curr->val, context)
-            + 2 * intpacksize;
-        }
-
+  listElem_t *r = resHList[nsp].resources;
+  for ( int i = 0; i < resHList[nsp].size; i++)
+    if (r[i].status & RESH_SYNC_BIT)
+      {
+        if (r[i].status == RESH_DESYNC_DELETED)
+          {
+            packBufferSize += 3 * intpacksize;
+          }
+        else if (r[i].status == RESH_DESYNC_IN_USE)
+          {
+            xassert ( r[i].res.v.ops );
+            /* message plus frame of 2 ints */
+            packBufferSize +=
+              r[i].res.v.ops->valGetPackSize(r[i].res.v.val, context)
+              + 2 * intpacksize;
+          }
+      }
   /* end marker */
   packBufferSize += intpacksize;
 
@@ -467,7 +583,6 @@ void reshPackBufferCreate(char **packBuffer, int *packBufferSize, void *context)
 {
   int i, packBufferPos = 0;
   int start = START, end = END, sep = SEPARATOR, type;
-  listElem_t * curr;
 
   xassert ( packBuffer );
 
@@ -476,33 +591,39 @@ void reshPackBufferCreate(char **packBuffer, int *packBufferSize, void *context)
   int nsp = namespaceGetActive ();
 
   int pBSize = *packBufferSize = getPackBufferSize(context);
-  char *pB = *packBuffer = xcalloc(1, *packBufferSize);
+  char *pB = *packBuffer = (char *)xcalloc(1, (size_t)pBSize);
 
   {
     int header[3] = { start, nsp, sep };
     serializePack(header, 3,  DATATYPE_INT, pB, pBSize, &packBufferPos, context);
   }
 
+  listElem_t *r = resHList[nsp].resources;
   for ( i = 0; i < resHList[nsp].size; i++ )
-    if ( resHList[nsp].resources[i].val )
-      if ( resHList[nsp].resources[i].status == ASSIGNED )
-        {
-          curr = resHList[nsp].resources + i;
-          xassert ( curr->ops );
-
-          type = curr->ops->valTxCode ();
-
-          if ( ! type ) continue;
-
-          serializePack( &type, 1, DATATYPE_INT, * packBuffer,
-			    * packBufferSize, &packBufferPos, context);
-
-          curr->ops->valPack(curr->val, pB, pBSize, &packBufferPos, context);
-
-          serializePack(&sep, 1, DATATYPE_INT, pB, pBSize, &packBufferPos, context);
-
-          curr->status = CLOSED;
-        }
+    if (r[i].status & RESH_SYNC_BIT)
+      {
+        if (r[i].status == RESH_DESYNC_DELETED)
+          {
+            enum { del_ints = 3 };
+            int temp[del_ints] = { RESH_DELETE, namespaceIdxEncode2(nsp, i), SEPARATOR };
+            serializePack(temp, del_ints, DATATYPE_INT,
+                          pB, pBSize, &packBufferPos, context);
+          }
+        else
+          {
+            listElem_t * curr = r + i;
+            xassert ( curr->res.v.ops );
+            type = curr->res.v.ops->valTxCode ();
+            if ( ! type ) continue;
+            serializePack(&type, 1, DATATYPE_INT, pB,
+                          pBSize, &packBufferPos, context);
+            curr->res.v.ops->valPack(curr->res.v.val,
+                                     pB, pBSize, &packBufferPos, context);
+            serializePack(&sep, 1, DATATYPE_INT,
+                          pB, pBSize, &packBufferPos, context);
+          }
+        r[i].status &= ~RESH_SYNC_BIT;
+      }
 
   LIST_UNLOCK();
 
@@ -513,15 +634,15 @@ void reshPackBufferCreate(char **packBuffer, int *packBufferSize, void *context)
 
 /* for thread safety this feature would have to be integrated in reshPut */
 
-void reshSetStatus ( cdiResH resH, resOps * ops, int status )
+void reshSetStatus ( cdiResH resH, const resOps * ops, int status )
 {
   int nsp;
   namespaceTuple_t nspT;
   listElem_t * listElem;
 
-  xassert ( ops );
+  xassert(ops && (status & RESH_IN_USE_BIT));
 
-  LIST_INIT();
+  LIST_INIT(1);
 
   LIST_LOCK();
 
@@ -536,7 +657,7 @@ void reshSetStatus ( cdiResH resH, resOps * ops, int status )
   listElem = resHList[nsp].resources + nspT.idx;
 
   xassert ( listElem &&
-            listElem->ops == ops );
+            listElem->res.v.ops == ops );
 
   listElem->status = status;
 
@@ -545,15 +666,14 @@ void reshSetStatus ( cdiResH resH, resOps * ops, int status )
 
 /**************************************************************/
 
-int reshGetStatus ( cdiResH resH, resOps * ops )
+int reshGetStatus ( cdiResH resH, const resOps * ops )
 {
   int nsp;
   namespaceTuple_t nspT;
-  listElem_t * listElem;
 
   xassert ( ops );
 
-  LIST_INIT();
+  LIST_INIT(1);
 
   LIST_LOCK();
 
@@ -565,12 +685,13 @@ int reshGetStatus ( cdiResH resH, resOps * ops )
             nspT.idx >= 0 &&
             nspT.idx < resHList[nsp].size );
 
-  listElem = resHList[nsp].resources + nspT.idx;
+  listElem_t *listElem = resHList[nsp].resources + nspT.idx;
+
+  const resOps *elemOps = listElem->res.v.ops;
 
   LIST_UNLOCK();
 
-  xassert ( listElem &&
-            listElem->ops == ops );
+  xassert(listElem && elemOps == ops);
 
   return listElem->status;
 }
@@ -593,7 +714,7 @@ void reshUnlock ()
 
 int reshListCompare ( int nsp0, int nsp1 )
 {
-  LIST_INIT();
+  LIST_INIT(1);
   LIST_LOCK();
 
   xassert(resHListSize > nsp0 && resHListSize > nsp1 &&
@@ -606,33 +727,34 @@ int reshListCompare ( int nsp0, int nsp1 )
     *resources1 = resHList[nsp1].resources;
   for (i = 0; i < listSizeMin; i++)
     {
-      int occupied0 = resources0[i].val != NULL,
-        occupied1 = resources1[i].val != NULL;
+      int occupied0 = (resources0[i].status & RESH_IN_USE_BIT) != 0,
+        occupied1 = (resources1[i].status & RESH_IN_USE_BIT) != 0;
       /* occupation mismatch ? */
       int diff = occupied0 ^ occupied1;
       valCompare |= (diff << cdiResHListOccupationMismatch);
       if (!diff && occupied0)
         {
           /* both occupied, do resource types match? */
-          diff = (resources0[i].ops != resources1[i].ops
-                  || resources0[i].ops == NULL);
+          diff = (resources0[i].res.v.ops != resources1[i].res.v.ops
+                  || resources0[i].res.v.ops == NULL);
           valCompare |= (diff << cdiResHListResourceTypeMismatch);
           if (!diff)
             {
               /* types match, does content match also? */
-              diff = resources0[i].ops->valCompare(resources0[i].val,
-                                                   resources1[i].val);
+              diff
+                = resources0[i].res.v.ops->valCompare(resources0[i].res.v.val,
+                                                      resources1[i].res.v.val);
               valCompare |= (diff << cdiResHListResourceContentMismatch);
             }
         }
     }
   /* find resources in nsp 0 beyond end of nsp 1 */
   for (int j = listSizeMin; j < resHList[nsp0].size; ++j)
-    valCompare |= ((resources0[j].val != NULL)
+    valCompare |= (((resources0[j].status & RESH_IN_USE_BIT) != 0)
                    << cdiResHListOccupationMismatch);
   /* find resources in nsp 1 beyond end of nsp 0 */
   for (; i < resHList[nsp1].size; ++i)
-    valCompare |= ((resources1[i].val != NULL)
+    valCompare |= (((resources1[i].status & RESH_IN_USE_BIT) != 0)
                    << cdiResHListOccupationMismatch);
 
   LIST_UNLOCK();
@@ -647,7 +769,7 @@ void reshListPrint(FILE *fp)
   int i, j, temp;
   listElem_t * curr;
 
-  LIST_INIT();
+  LIST_INIT(1);
 
 
   temp = namespaceGetActive ();
@@ -671,9 +793,9 @@ void reshListPrint(FILE *fp)
       for ( j = 0; j < resHList[i].size; j++ )
         {
           curr = resHList[i].resources + j;
-          if ( curr->ops && curr->val )
+          if (!(curr->status & RESH_IN_USE_BIT))
             {
-              curr->ops->valPrint (( void * ) curr->val, fp );
+              curr->res.v.ops->valPrint(curr->res.v.val, fp);
               fprintf ( fp, "\n" );
             }
         }
