@@ -33,19 +33,37 @@ typedef struct {
   valTxCodeFunc      valTxCode;
 }resOps;
 
-enum { RESH_UNDEFID, ASSIGNED, SUSPENDED, CLOSED };
+enum {
+  RESH_IN_USE_BIT = 1 << 0,
+  RESH_SYNC_BIT = 1 << 1,
+  /* resource holds no value */
+  RESH_UNUSED = 0,
+  /* resource was deleted and needs to be synced */
+  RESH_DESYNC_DELETED
+    = RESH_SYNC_BIT,
+  /* resource is synchronized */
+  RESH_IN_USE
+    = RESH_IN_USE_BIT,
+  /* resource is in use, desynchronized and needs to be synced */
+  RESH_DESYNC_IN_USE
+    = RESH_IN_USE_BIT | RESH_SYNC_BIT,
+};
 
 void   reshListCreate(int namespaceID);
 void   reshListDestruct(int namespaceID);
-int    reshPut ( void *, resOps * );
-void   reshRemove ( cdiResH, resOps * );
+int    reshPut ( void *, const resOps * );
+void reshReplace(cdiResH resH, void *p, const resOps *ops);
+void   reshRemove ( cdiResH, const resOps * );
+/*> doesn't check resource type */
+void reshDestroy(cdiResH);
 
-int    reshCountType ( resOps * );
+int    reshCountType ( const resOps * );
 
-void * reshGetValue(const char *, cdiResH, resOps * );
-#define reshGetVal(resH, ops)  reshGetValue(__func__, resH, ops)
+void * reshGetValue(const char* caller, const char* expressionString, cdiResH id, const resOps* ops);
+#define reshGetVal(resH, ops)  reshGetValue(__func__, #resH, resH, ops)
 
-void   reshGetResHListOfType ( int, int *, resOps * );
+
+void   reshGetResHListOfType ( int, int *, const resOps * );
 
 enum cdiApplyRet {
   CDI_APPLY_ERROR = -1,
@@ -63,11 +81,13 @@ cdiResHFilterApply(const resOps *p,
 
 void   reshPackBufferCreate ( char **, int *, void *context );
 void   reshPackBufferDestroy ( char ** );
-int    reshResourceGetPackSize(int resh, resOps *ops, void *context);
-void   reshPackResource(int resh, resOps *ops,
-                        void *buf, int buf_size, int *position, void *context);
-void   reshSetStatus ( cdiResH, resOps *, int );
-int    reshGetStatus ( cdiResH, resOps * );
+int    reshResourceGetPackSize_intern(int resh, const resOps *ops, void *context, const char* caller, const char* expressionString);
+#define reshResourceGetPackSize(resh, ops, context) reshResourceGetPackSize_intern(resh, ops, context, __func__, #resh)
+void   reshPackResource_intern(int resh, const resOps *ops, void *buf, int buf_size, int *position, void *context, const char* caller, const char* expressionString);
+#define reshPackResource(resh, ops, buf, buf_size, position, context) reshPackResource_intern(resh, ops, buf, buf_size, position, context, __func__, #resh)
+
+void   reshSetStatus ( cdiResH, const resOps *, int );
+int    reshGetStatus ( cdiResH, const resOps * );
 
 void   reshLock   ( void );
 void   reshUnlock ( void );
