@@ -19,25 +19,22 @@
 
 void recordInitEntry(record_t *record)
 {
-  (*record).position = CDI_UNDEFID;
-  (*record).size     = 0;
-  (*record).param    = 0;
-  (*record).ilevel   = CDI_UNDEFID;
-  (*record).used     = FALSE;
-  (*record).varID    = CDI_UNDEFID;
-  (*record).levelID  = CDI_UNDEFID;
-  memset((*record).varname, 0, sizeof((*record).varname));
+  record->position = CDI_UNDEFID;
+  record->size     = 0;
+  record->param    = 0;
+  record->ilevel   = CDI_UNDEFID;
+  record->used     = FALSE;
+  record->varID    = CDI_UNDEFID;
+  record->levelID  = CDI_UNDEFID;
+  memset(record->varname, 0, sizeof(record->varname));
 }
 
 
 int recordNewEntry(stream_t *streamptr, int tsID)
 {
   int recordID = 0;
-  int recordSize;
-  record_t *records;
-
-  recordSize = streamptr->tsteps[tsID].recordSize;
-  records    = streamptr->tsteps[tsID].records;
+  int recordSize = streamptr->tsteps[tsID].recordSize;
+  record_t *records = streamptr->tsteps[tsID].records;
   /*
     Look for a free slot in record.
     (Create the table the first time through).
@@ -46,7 +43,7 @@ int recordNewEntry(stream_t *streamptr, int tsID)
     {
       int i;
       recordSize = 1;   /*  <<<<----  */
-      records = (record_t *) malloc(recordSize*sizeof(record_t));
+      records = (record_t *)xmalloc((size_t)recordSize * sizeof (record_t));
       if ( records == NULL )
 	{
           Message("recordSize = %d", recordSize);
@@ -72,7 +69,8 @@ int recordNewEntry(stream_t *streamptr, int tsID)
       int i;
 
       recordSize = 2*recordSize;
-      records    = (record_t *) realloc(records, recordSize*sizeof(record_t));
+      records    = (record_t *)xrealloc(records,
+                                        (size_t)recordSize * sizeof (record_t));
       if ( records == NULL )
 	{
           Message("recordSize = %d", recordSize);
@@ -120,95 +118,46 @@ void cdiInitRecord(stream_t *streamptr)
 
 void streamInqRecord(int streamID, int *varID, int *levelID)
 {
-  /* int rec = 0; */
-  int recID, tsID, rindex;
-  int lindex;
-  stream_t *streamptr;
-
   check_parg(varID);
   check_parg(levelID);
 
-  streamptr = stream_to_pointer(streamID);
-
+  stream_t *streamptr = stream_to_pointer(streamID);
   stream_check_ptr(__func__, streamptr);
 
   cdiDefAccesstype(streamID, TYPE_REC);
 
   if ( ! streamptr->record ) cdiInitRecord(streamptr);
 
-  tsID   = streamptr->curTsID;
-  rindex = streamptr->tsteps[tsID].curRecID + 1;
+  int tsID   = streamptr->curTsID;
+  int rindex = streamptr->tsteps[tsID].curRecID + 1;
 
   if ( rindex >= streamptr->tsteps[tsID].nrecs )
     Error("record %d not available at timestep %d", rindex+1, tsID+1);
 
-  recID  = streamptr->tsteps[tsID].recIDs[rindex];
+  int recID  = streamptr->tsteps[tsID].recIDs[rindex];
 
   if ( recID == -1 || recID >= streamptr->tsteps[tsID].nallrecs )
     Error("Internal problem! tsID = %d recID = %d", tsID, recID);
 
   *varID   = streamptr->tsteps[tsID].records[recID].varID;
-  lindex   = streamptr->tsteps[tsID].records[recID].levelID;
+  int lindex = streamptr->tsteps[tsID].records[recID].levelID;
 
   *levelID = streamptr->vars[*varID].lindex[lindex];
 
   if ( CDI_Debug )
-    Message("tsID = %d, recID = %d, varID = %d, levelID = %d\n",
-	    tsID, recID, *varID, *levelID);
+    Message("tsID = %d, recID = %d, varID = %d, levelID = %d\n", tsID, recID, *varID, *levelID);
 
   streamptr->curTsID = tsID;
   streamptr->tsteps[tsID].curRecID = rindex;
-
-  /*
-  rec = recID + 1;
-  filetype = streamptr->filetype;
-
-  switch ( filetype )
-    {
-    case FILETYPE_GRB:
-      {
-        rec = grbInqRecord(streamID, varID, levelID);
-	break;
-      }
-    case FILETYPE_SRV:
-      {
-        rec = srvInqRecord(streamptr, varID, levelID);
-	break;
-      }
-#if  defined  (HAVE_LIBNETCDF)
-    case FILETYPE_NC:
-    case FILETYPE_NC2:
-    case FILETYPE_NC4:
-    case FILETYPE_NC4C:
-      {
-	rec = cdfInqRecord(streamID, varID, levelID);
-	break;
-      }
-#endif
-    default:
-      {
-	Error("%s support not compiled in!", strfiletype(filetype));
-	break;
-      }
-    }
-  */
 }
 
 
 void streamDefRecord(int streamID, int varID, int levelID)
 {
-  int status = 0;
-  int filetype;
-  int param, gridID, zaxisID, level;
-  int tsID;
-  int vlistID;
-  stream_t *streamptr;
-
-  streamptr = stream_to_pointer(streamID);
-
+  stream_t *streamptr = stream_to_pointer(streamID);
   stream_check_ptr(__func__, streamptr);
 
-  tsID = streamptr->curTsID;
+  int tsID = streamptr->curTsID;
 
   if ( tsID == CDI_UNDEFID )
     {
@@ -218,11 +167,11 @@ void streamDefRecord(int streamID, int varID, int levelID)
 
   if ( ! streamptr->record ) cdiInitRecord(streamptr);
 
-  vlistID = streamptr->vlistID;
-  gridID  = vlistInqVarGrid(vlistID, varID);
-  zaxisID = vlistInqVarZaxis(vlistID, varID);
-  param   = vlistInqVarParam(vlistID, varID);
-  level   = (int) zaxisInqLevel(zaxisID, levelID);
+  int vlistID = streamptr->vlistID;
+  int gridID  = vlistInqVarGrid(vlistID, varID);
+  int zaxisID = vlistInqVarZaxis(vlistID, varID);
+  int param   = vlistInqVarParam(vlistID, varID);
+  int level   = (int) zaxisInqLevel(zaxisID, levelID);
 
   streamptr->record->varID    = varID;
   streamptr->record->levelID  = levelID;
@@ -234,182 +183,127 @@ void streamDefRecord(int streamID, int varID, int levelID)
   streamptr->record->zaxisID  = zaxisID;
   streamptr->record->prec     = vlistInqVarDatatype(vlistID, varID);
 
-  filetype = streamptr->filetype;
-
-  switch (filetype)
+  switch (streamptr->filetype)
     {
 #if  defined  (HAVE_LIBGRIB)
     case FILETYPE_GRB:
     case FILETYPE_GRB2:
-      {
-        /* FIXME: return value not inspected */
-        status = grbDefRecord(streamptr);
-	break;
-      }
+      grbDefRecord(streamptr);
+      break;
 #endif
 #if  defined  (HAVE_LIBSERVICE)
     case FILETYPE_SRV:
-      {
-        /* FIXME: return value not inspected */
-        status = srvDefRecord(streamptr);
-	break;
-      }
+      srvDefRecord(streamptr);
+      break;
 #endif
 #if  defined  (HAVE_LIBEXTRA)
     case FILETYPE_EXT:
-      {
-        /* FIXME: return value not inspected */
-        status = extDefRecord(streamptr);
-	break;
-      }
+      extDefRecord(streamptr);
+      break;
 #endif
 #if  defined  (HAVE_LIBIEG)
     case FILETYPE_IEG:
-      {
-        /* FIXME: return value not inspected */
-        status = iegDefRecord(streamptr);
-	break;
-      }
+      iegDefRecord(streamptr);
+      break;
 #endif
 #if  defined  (HAVE_LIBNETCDF)
     case FILETYPE_NC:
     case FILETYPE_NC2:
     case FILETYPE_NC4:
     case FILETYPE_NC4C:
-      {
-	if ( streamptr->accessmode == 0 ) cdfEndDef(streamptr);
-	status = cdfDefRecord(streamptr);
-	break;
-      }
+      if ( streamptr->accessmode == 0 ) cdfEndDef(streamptr);
+      cdfDefRecord(streamptr);
+      break;
 #endif
     default:
-      {
-	Error("%s support not compiled in!", strfiletype(filetype));
-	break;
-      }
+      Error("%s support not compiled in!", strfiletype(streamptr->filetype));
+      break;
     }
 }
 
 
 void streamReadRecord(int streamID, double *data, int *nmiss)
 {
-  int status = 0;
-  int filetype;
-  stream_t *streamptr;
-
   check_parg(data);
   check_parg(nmiss);
 
-  streamptr = stream_to_pointer(streamID);
-
+  stream_t *streamptr = stream_to_pointer(streamID);
   stream_check_ptr(__func__, streamptr);
 
-  filetype = streamptr->filetype;
-
   *nmiss = 0;
 
-  switch (filetype)
+  switch (streamptr->filetype)
     {
 #if  defined  (HAVE_LIBGRIB)
     case FILETYPE_GRB:
     case FILETYPE_GRB2:
-      {
-        /* FIXME: return value not inspected */
-        status = grbReadRecord(streamptr, data, nmiss);
-	break;
-      }
+      grbReadRecord(streamptr, data, nmiss);
+      break;
 #endif
 #if  defined  (HAVE_LIBSERVICE)
     case FILETYPE_SRV:
-      {
-        /* FIXME: return value not inspected */
-        status = srvReadRecord(streamptr, data, nmiss);
-	break;
-      }
+      srvReadRecord(streamptr, data, nmiss);
+      break;
 #endif
 #if  defined  (HAVE_LIBEXTRA)
     case FILETYPE_EXT:
-      {
-        /* FIXME: return value not inspected */
-        status = extReadRecord(streamptr, data, nmiss);
-	break;
-      }
+      extReadRecord(streamptr, data, nmiss);
+      break;
 #endif
 #if  defined  (HAVE_LIBIEG)
     case FILETYPE_IEG:
-      {
-        /* FIXME: return value not inspected */
-        status = iegReadRecord(streamptr, data, nmiss);
-	break;
-      }
+      iegReadRecord(streamptr, data, nmiss);
+      break;
 #endif
 #if  defined  (HAVE_LIBNETCDF)
     case FILETYPE_NC:
     case FILETYPE_NC2:
     case FILETYPE_NC4:
     case FILETYPE_NC4C:
-      {
-        /* FIXME: return value not inspected */
-	status = cdfReadRecord(streamptr, data, nmiss);
-	break;
-      }
+      cdfReadRecord(streamptr, data, nmiss);
+      break;
 #endif
     default:
       {
-	Error("%s support not compiled in!", strfiletype(filetype));
+	Error("%s support not compiled in!", strfiletype(streamptr->filetype));
 	break;
       }
     }
 }
 
-
-void stream_write_record(int streamID, int memtype, const void *data, int nmiss)
+static void
+stream_write_record(int streamID, int memtype, const void *data, int nmiss)
 {
-  int status = 0;
-  int filetype;
-  stream_t *streamptr;
-
   check_parg(data);
 
-  streamptr = stream_to_pointer(streamID);
-
+  stream_t *streamptr = stream_to_pointer(streamID);
   stream_check_ptr(__func__, streamptr);
 
-  filetype = streamptr->filetype;
-
-  switch (filetype)
+  switch (streamptr->filetype)
     {
 #if  defined  (HAVE_LIBGRIB)
     case FILETYPE_GRB:
     case FILETYPE_GRB2:
-      {
-        status = grb_write_record(streamptr, memtype, data, nmiss);
-	break;
-      }
+      grb_write_record(streamptr, memtype, data, nmiss);
+      break;
 #endif
 #if  defined  (HAVE_LIBSERVICE)
     case FILETYPE_SRV:
-      {
-        if ( memtype == MEMTYPE_FLOAT ) Error("srvWriteRecord not implemented for memtype float!");
-        status = srvWriteRecord(streamptr, data);
-	break;
-      }
+      if ( memtype == MEMTYPE_FLOAT ) Error("srvWriteRecord not implemented for memtype float!");
+      srvWriteRecord(streamptr, (const double *)data);
+      break;
 #endif
 #if  defined  (HAVE_LIBEXTRA)
     case FILETYPE_EXT:
-      {
-        if ( memtype == MEMTYPE_FLOAT ) Error("extWriteRecord not implemented for memtype float!");
-        status = extWriteRecord(streamptr, data);
-	break;
-      }
+      if ( memtype == MEMTYPE_FLOAT ) Error("extWriteRecord not implemented for memtype float!");
+      extWriteRecord(streamptr, (const double *)data);
+      break;
 #endif
 #if  defined  (HAVE_LIBIEG)
     case FILETYPE_IEG:
-      {
-        if ( memtype == MEMTYPE_FLOAT ) Error("iegWriteRecord not implemented for memtype float!");
-        status = iegWriteRecord(streamptr, data);
-	break;
-      }
+      if ( memtype == MEMTYPE_FLOAT ) Error("iegWriteRecord not implemented for memtype float!");
+      iegWriteRecord(streamptr, (const double *)data);
+      break;
 #endif
 #if  defined  (HAVE_LIBNETCDF)
     case FILETYPE_NC:
@@ -423,7 +317,7 @@ void stream_write_record(int streamID, int memtype, const void *data, int nmiss)
 #endif
     default:
       {
-	Error("%s support not compiled in!", strfiletype(filetype));
+	Error("%s support not compiled in!", strfiletype(streamptr->filetype));
 	break;
       }
     }
@@ -443,19 +337,15 @@ void streamWriteRecordF(int streamID, const float *data, int nmiss)
 
 void streamCopyRecord(int streamID2, int streamID1)
 {
-  int status = 0;
-  int filetype = CDI_UNDEFID, filetype1, filetype2;
-  stream_t *streamptr1;
-  stream_t *streamptr2;
-
-  streamptr1 = stream_to_pointer(streamID1);
-  streamptr2 = stream_to_pointer(streamID2);
+  stream_t *streamptr1 = stream_to_pointer(streamID1);
+  stream_t *streamptr2 = stream_to_pointer(streamID2);
 
   stream_check_ptr(__func__, streamptr1);
   stream_check_ptr(__func__, streamptr2);
 
-  filetype1 = streamptr1->filetype;
-  filetype2 = streamptr2->filetype;
+  int filetype1 = streamptr1->filetype;
+  int filetype2 = streamptr2->filetype;
+  int filetype  = FILETYPE_UNDEF;
 
   if ( filetype1 == filetype2 ) filetype = filetype2;
   else
@@ -480,7 +370,7 @@ void streamCopyRecord(int streamID2, int streamID1)
         }
     }
 
-  if ( filetype == CDI_UNDEFID )
+  if ( filetype == FILETYPE_UNDEF )
     Error("Streams have different file types (%s -> %s)!", strfiletype(filetype1), strfiletype(filetype2));
 
   switch (filetype)
@@ -488,46 +378,31 @@ void streamCopyRecord(int streamID2, int streamID1)
 #if  defined  (HAVE_LIBGRIB)
     case FILETYPE_GRB:
     case FILETYPE_GRB2:
-      {
-        /* FIXME: return value not inspected */
-	status = grbCopyRecord(streamptr2, streamptr1);
-	break;
-      }
+      grbCopyRecord(streamptr2, streamptr1);
+      break;
 #endif
 #if  defined  (HAVE_LIBSERVICE)
     case FILETYPE_SRV:
-      {
-        /* FIXME: return value not inspected */
-	status = srvCopyRecord(streamptr2, streamptr1);
-	break;
-      }
+      srvCopyRecord(streamptr2, streamptr1);
+      break;
 #endif
 #if  defined  (HAVE_LIBEXTRA)
     case FILETYPE_EXT:
-      {
-        /* FIXME: return value not inspected */
-	status = extCopyRecord(streamptr2, streamptr1);
-	break;
-      }
+      extCopyRecord(streamptr2, streamptr1);
+      break;
 #endif
 #if  defined  (HAVE_LIBIEG)
     case FILETYPE_IEG:
-      {
-        /* FIXME: return value not inspected */
-	status = iegCopyRecord(streamptr2, streamptr1);
-	break;
-      }
+      iegCopyRecord(streamptr2, streamptr1);
+      break;
 #endif
 #if  defined  (HAVE_LIBNETCDF)
     case FILETYPE_NC:
     case FILETYPE_NC2:
     case FILETYPE_NC4:
     case FILETYPE_NC4C:
-      {
-        /* FIXME: return value not inspected */
-	status = cdfCopyRecord(streamptr2, streamptr1);
-	break;
-      }
+      cdfCopyRecord(streamptr2, streamptr1);
+      break;
 #endif
     default:
       {
@@ -540,24 +415,22 @@ void streamCopyRecord(int streamID2, int streamID1)
 
 void cdi_create_records(stream_t *streamptr, int tsID)
 {
-  int nrecords, maxrecords;
-  int nvars, varID, recID;
+  unsigned nrecords, maxrecords;
   record_t *records;
-  int vlistID;
 
   if ( streamptr->tsteps[tsID].records ) return;
 
-  vlistID  = streamptr->vlistID;
+  int vlistID  = streamptr->vlistID;
 
   if ( tsID == 0 )
     {
       maxrecords = 0;
-      nvars = streamptr->nvars;
-      for ( varID = 0; varID < nvars; varID++)
-	maxrecords += streamptr->vars[varID].nlevs;
+      int nvars = streamptr->nvars;
+      for ( int varID = 0; varID < nvars; varID++)
+	maxrecords += (unsigned)streamptr->vars[varID].nlevs;
     }
   else
-    maxrecords = streamptr->tsteps[0].recordSize;
+    maxrecords = (unsigned)streamptr->tsteps[0].recordSize;
 
   if ( tsID == 0 )
     {
@@ -566,42 +439,41 @@ void cdi_create_records(stream_t *streamptr, int tsID)
   else if ( tsID == 1 )
     {
       nrecords = 0;
-      maxrecords = streamptr->tsteps[0].recordSize;
-      for ( recID = 0; recID < maxrecords; recID++ )
+      maxrecords = (unsigned)streamptr->tsteps[0].recordSize;
+      for (unsigned recID = 0; recID < maxrecords; recID++ )
 	{
-	  varID = streamptr->tsteps[0].records[recID].varID;
-	  if ( varID != -1 ) /* varID = -1 for write mode !!! */
-	    if ( vlistInqVarTsteptype(vlistID, varID) == TSTEP_CONSTANT )
-	      continue;
-	  nrecords++;
+	  int varID = streamptr->tsteps[0].records[recID].varID;
+	  nrecords +=
+            (varID == -1 /* varID = -1 for write mode !!! */
+             || vlistInqVarTsteptype(vlistID, varID) != TSTEP_CONSTANT);
 	}
     }
   else
-    nrecords = streamptr->tsteps[1].nallrecs;
+    nrecords = (unsigned)streamptr->tsteps[1].nallrecs;
 
   if ( maxrecords > 0 )
-    records = (record_t *) malloc(maxrecords*sizeof(record_t));
+    records = (record_t *) malloc(maxrecords * sizeof(record_t));
   else
     records = NULL;
 
   streamptr->tsteps[tsID].records    = records;
-  streamptr->tsteps[tsID].recordSize = maxrecords;
-  streamptr->tsteps[tsID].nallrecs   = nrecords;
+  streamptr->tsteps[tsID].recordSize = (int)maxrecords;
+  streamptr->tsteps[tsID].nallrecs   = (int)nrecords;
 
   if ( tsID == 0 )
     {
-      for ( recID = 0; recID < maxrecords; recID++ )
+      for ( unsigned recID = 0; recID < maxrecords; recID++ )
 	recordInitEntry(&streamptr->tsteps[tsID].records[recID]);
     }
   else
     {
       memcpy(streamptr->tsteps[tsID].records,
 	     streamptr->tsteps[0].records,
-	     maxrecords*sizeof(record_t));
+	     (size_t)maxrecords * sizeof(record_t));
 
-      for ( recID = 0; recID < maxrecords; recID++ )
+      for ( unsigned recID = 0; recID < maxrecords; recID++ )
 	{
-	  varID = streamptr->tsteps[0].records[recID].varID;
+	  int varID = streamptr->tsteps[0].records[recID].varID;
 	  if ( varID != -1 ) /* varID = -1 for write mode !!! */
 	    if ( vlistInqVarTsteptype(vlistID, varID) != TSTEP_CONSTANT )
 	      {
